require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xazurepackadmin',
  dscmeta_resource_friendly_name: 'xAzurePackAdmin',
  dscmeta_resource_name: 'MSFT_xAzurePackAdmin',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xAzurePack',
  dscmeta_module_version: '1.4.0.0',
  docs: 'The DSC xAzurePackAdmin resource type.
         Automatically generated from version 1.4.0.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_dbuser: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'SQL user to be used to create the database if the SetupCredential cannot be used.',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_sqlinstance: {
      type: 'Optional[String]',
      desc: 'Database instance for the Azure Pack databases.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_sqlserver: {
      type: 'String',
      desc: 'Database server for the Azure Pack databases.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_azurepackadmincredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Credential to be used to perform the installation.',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_principal: {
      type: 'String',
      desc: 'The Azure Pack admin principal.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_ensure: {
      type: "Optional[Enum['Present', 'Absent']]",
      desc: 'An enumerated value that describes if the principal is an Azure Pack admin. Present {default}   Absent    ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
