require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xazurepackresourceprovider',
  dscmeta_resource_friendly_name: 'xAzurePackResourceProvider',
  dscmeta_resource_name: 'MSFT_xAzurePackResourceProvider',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xAzurePack',
  dscmeta_module_version: '1.4.0.0',
  docs: 'The DSC xAzurePackResourceProvider resource type.
         Automatically generated from version 1.4.0.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_notificationauthenticationuser: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Specifies, as a PSCredential object, a notification user name and password to connect to a resource provider.',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_adminauthenticationmode: {
      type: "Optional[Enum['None', 'Basic', 'Windows']]",
      desc: 'Specifies the administrative authentication mode for a resource provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_allowanonymousaccess: {
      type: 'Optional[Boolean]',
      desc: 'Specifies the URI of the Windows Azure Pack administrator API.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_allowmultipleinstances: {
      type: 'Optional[Boolean]',
      desc: 'Indicates that the cmdlet allows multiple instances of the resource provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_displayname: {
      type: 'Optional[String]',
      desc: 'Specifies the display name of a resource provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_enabled: {
      type: 'Optional[Boolean]',
      desc: 'Enables the resource provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_usageauthenticationuser: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Specifies, as a PSCredential object, a usage user name and password to connect to a resource provider.',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_name: {
      type: 'String',
      desc: 'Specifies the name of a resource provider.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_authenticationsite: {
      type: 'String',
      desc: 'URL of the authentication site.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_healthcheckauthenticationusername: {
      type: 'Optional[String]',
      desc: 'Output for the health check user name.',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_usageforwardingaddress: {
      type: 'Optional[String]',
      desc: 'Specifies the tenant forwarding address of a resource provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_passthroughenabled: {
      type: 'Optional[Boolean]',
      desc: 'Indicates whether the resource provider supports API pass-through.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_usageauthenticationusername: {
      type: 'Optional[String]',
      desc: 'Output for the usage user name.',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_adminauthenticationuser: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Specifies, as a PSCredential object, an administrative user name and password to connect to a resource provider.',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_tenanttargeturitemplate: {
      type: 'Optional[String]',
      desc: 'Specifies the tenant target URI template of a resource provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_usageauthenticationmode: {
      type: "Optional[Enum['None', 'Basic', 'Windows']]",
      desc: 'Specifies the usage authentication mode for a resource provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_tenantauthenticationmode: {
      type: "Optional[Enum['None', 'Basic', 'Windows']]",
      desc: 'Specifies the tenant authentication mode for a resource provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_adminforwardingaddress: {
      type: 'Optional[String]',
      desc: 'Specifies an administrative forwarding address for a resource provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_healthcheckauthenticationmode: {
      type: "Optional[Enum['None', 'Basic', 'Windows']]",
      desc: 'Specifies the health check authentication mode for a resource provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_tenantforwardingaddress: {
      type: 'Optional[String]',
      desc: 'Specifies the tenant forwarding address of a resource provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_notificationauthenticationmode: {
      type: "Optional[Enum['None', 'Basic', 'Windows']]",
      desc: 'Specifies the notification authentication mode for a resource provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_tenantsourceuritemplate: {
      type: 'Optional[String]',
      desc: 'Specifies the tenant source URI template of a resource provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_azurepackadmincredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Credential to be used to perform the installation.',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_instanceid: {
      type: 'Optional[String]',
      desc: 'Specifies an ID for an instance of a resource provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_type: {
      type: "Optional[Enum['Standard', 'UsageProvider', 'CloudServiceProvider']]",
      desc: 'Specifies the type of the resource provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_tenantauthenticationusername: {
      type: 'Optional[String]',
      desc: 'Output for the tenant user name.',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_notificationforwardingaddress: {
      type: 'Optional[String]',
      desc: 'Specifies the notification forwarding address of a resource provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_adminauthenticationusername: {
      type: 'Optional[String]',
      desc: 'Output for the administrative user name.',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_subscriptionstatuspollinginterval: {
      type: 'Optional[String]',
      desc: 'Specifies the time interval at which the management service polls the resource provider for subscription status updates.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_adminuri: {
      type: 'String',
      desc: 'Specifies the URI of the Windows Azure Pack administrator API.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_healthcheckforwardingaddress: {
      type: 'Optional[String]',
      desc: 'Specifies the health check forwarding address for a resource provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_instancedisplayname: {
      type: 'Optional[String]',
      desc: 'Specifies a display name for an instance of a resource provider.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_tenantauthenticationuser: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Specifies, as a PSCredential object, a tenant user name and password to connect to a resource provider.',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_notificationauthenticationusername: {
      type: 'Optional[String]',
      desc: 'Output for the notification user name.',
      behaviour: :read_only,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_healthcheckauthenticationuser: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Specifies, as a PSCredential object, a health check user name and password to connect to a resource provider.',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_maxquotaupdatebatchsize: {
      type: 'Optional[String]',
      desc: 'Specifies the number of subscriptions that can be updated in a single request.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
