require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xazurepacksetup',
  dscmeta_resource_friendly_name: 'xAzurePackSetup',
  dscmeta_resource_name: 'MSFT_xAzurePackSetup',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xAzurePack',
  dscmeta_module_version: '1.4.0.0',
  docs: 'The DSC xAzurePackSetup resource type.
         Automatically generated from version 1.4.0.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_sourcepath: {
      type: 'String',
      desc: 'UNC path to the root of the source files for installation.',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sqlinstance: {
      type: 'Optional[String]',
      desc: 'Database instance for the Azure Pack databases.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_action: {
      type: "Enum['Install', 'Initialize']",
      desc: 'Install or initialize.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_dbuser: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'SQL user to be used to create the database if the SetupCredential cannot be used.',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_setupcredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Credential to be used to perform the installation.',
      behaviour: :parameter,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_passphrase: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: 'Passphrase for the Azure Pack deployment.',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_role: {
      type: "Enum['Admin API', 'Tenant API', 'Tenant Public API', 'SQL Server Extension', 'MySQL Extension', 'Admin Site', 'Admin Authentication Site', 'Tenant Site', 'Tenant Authentication Site']",
      desc: 'The Azure Pack role to be installed or initialized.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sqlserver: {
      type: 'Optional[String]',
      desc: 'Database server for the Azure Pack databases.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_enableceip: {
      type: 'Optional[String]',
      desc: 'Enable Customer Experience Improvement Program.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_sourcefolder: {
      type: 'Optional[String]',
      desc: 'Folder within the source path containing the source files for installation.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
